import 'dart:io' show NetworkInterface;
import 'package:permission_handler/permission_handler.dart';
import 'package:platform_device_id/platform_device_id.dart';
import 'package:intl/intl.dart';
import 'package:intl/date_symbol_data_local.dart';
import '../import_base_lib.dart';
import '../utils/import_lib_util_time.dart' as timeago;

class CoreUtil {
  static Future<String> getIP() {
    return NetworkInterface.list().then((value) {
      for (var interface in value.toList()) {
        for (var item in interface.addresses) {
          if (item.address.isNotEmpty) return item.address;
        }
      }
      return '192.168.1.10';
    });
  }

  static String stringToLinkHtml(String str, {double width = 0.0, double height = 0.0}) {
    String result = str;
    if (str.length > 2) {
      String tmp = str;
      RegExp exp = RegExp(CoreConstants.patternLinkHtml);
      Iterable<RegExpMatch> matches = exp.allMatches(str.toLowerCase());
      for (var match in matches) {
        String url = tmp.substring(match.start, match.end);
        if (url.contains('..') || double.tryParse(url) != null) {continue;}
        else {
          String symbol = '';
          if (match.start > 0) symbol = tmp.substring(match.start - 1, match.start);
          if (!symbol.contains('#') && url.length > 3) {
            String http = url;
            if (!url.toLowerCase().contains('http')) http = 'https://$url';
            result = result.replaceFirst(
              url,
              "<a href='$http'>$url</a>",
            );
          }
        }
      }
    }
    return result;
  }

  static Future<PermissionStatus> checkPermission(Permission permission) async {
    PermissionStatus status = await permission.status;
    if (status == PermissionStatus.denied ||
        status == PermissionStatus.limited) {
      return permission.request();
    } else if (status == PermissionStatus.restricted ||
        status == PermissionStatus.permanentlyDenied) { openAppSettings(); }
    return status;
  }

  static String getFullAddress(String address, String city, String district) {
    String temp = address;
    if (district.isNotEmpty) temp.isNotEmpty ? temp += ', $district' : temp = district;
    if (city.isNotEmpty) temp.isNotEmpty ? temp += ', $city' : temp = city;
    return temp;
  }

  static dynamic getValueFromJson(json, String key, dynamic defaultValue) {
    if (json.containsKey(key) && json[key] != null) return json[key];
    return defaultValue;
  }

  static bool checkKeyFromJson(json, String key) =>
      json.containsKey(key) && json[key] != null && json.isNotEmpty;

  static bool isImage(String url) =>
      url.contains('.jpg') ||
      url.contains('.jpeg') ||
      url.contains('.png') ||
      url.contains('.gif') ||
      url.contains('.ico') ||
      url.contains('.svg');

  static bool isVideo(String url) =>
      url.contains('.mp4') ||
      url.contains('.ogv') ||
      url.contains('.webm') ||
      url.contains('.3gp') ||
      url.contains('.m3u8') ||
      url.contains('.avi') ||
      url.contains('.mov');

  static bool isAudio(String url) =>
      url.contains('.mp3') ||
      url.contains('.ogg') ||
      url.contains('.wma') ||
      url.contains('.m4a') ||
      url.contains('.wav');

  static String doubleToString(double value,
      {String locale = CoreConstants.localeENLang, int? digits}) {
    int decimalDigits = CoreConstants.decimalDigitsEN;
    if (locale == CoreConstants.localeVILang) decimalDigits = CoreConstants.decimalDigitsVI;

    digits ??= decimalDigits;

    String tmp = value.toString();
    try {
      final nfc = NumberFormat.currency(locale: locale, decimalDigits: digits);
      tmp = nfc.format(value).replaceAll(RegExp(nfc.currencyName!), '').trim();
    } catch (_) {}
    return tmp;
  }

  static String dateToString(DateTime date,
      {String pattern = CoreConstants.datePatternEN,
      String locale = CoreConstants.localeEN}) {
    initializeDateFormatting(locale, null);
    return DateFormat(pattern, locale).format(date);
  }

  static DateTime stringToDateTime(String source,
      {String pattern = CoreConstants.dateTimePattern,
      String locale = CoreConstants.localeEN}) {
    try {
      initializeDateFormatting(locale, null);
      return DateFormat(pattern, locale).parse(source);
    } catch (_) { return DateTime.now(); }
  }

  static String getTimeAgo(String strDatetime) {
    final now = DateTime.now();
    final ago = CoreUtil.stringToDateTime(strDatetime);
    final dua = now.difference(ago);
    if (CoreConstants.locale == CoreConstants.localeEN) {
      return timeago.format(now.subtract(dua));
    } else {
      timeago.setLocaleMessages(CoreConstants.localeVILang, timeago.ViMessages());
      return timeago.format(now.subtract(dua), locale: CoreConstants.localeVILang);
    }
  }

  static Future<String> getDeviceId() async {
    String imei = '';
    try {
      imei = (await PlatformDeviceId.getDeviceId)!;
    } catch (e) {
      SharedPreferences prefs = await SharedPreferences.getInstance();
      if (prefs.containsKey(CoreConstants.deviceId)) imei = prefs.getString(CoreConstants.deviceId)!;
    }
    return imei;
  }
}
