import 'dart:async';
import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:url_launcher/url_launcher.dart';
import '../api_client.dart';
import '../base_response.dart';
import '../core_constants.dart';
import '../multi_language.dart';
import '../ui/button_custom_transparent.dart';
import '../ui/text_custom.dart';

class CoreUtilUI {
  static void viewAvatar(BuildContext context,
      {String assetDefault = 'assets/images/ic_default_avatar.png', EdgeInsets? padding}) {
    showDialog(context: context,
        builder: (context) => Container(padding: padding??EdgeInsets.fromLTRB(
            10.sp, ScreenUtil().statusBarHeight, 10.sp, ScreenUtil().bottomBarHeight),
            alignment: Alignment.center,
            child: FadeInImage.assetNetwork(
                image: ApiClient().baseUrl + CoreConstants.avatar, fit: BoxFit.cover,
                placeholder: assetDefault,
                imageErrorBuilder: (context, obj, stack) => Image.asset(assetDefault,
                    fit: BoxFit.fill))
        )
    );
  }

  static void launchUrl(BuildContext context, String url) => canLaunch(url).then((value) {
    value ? launch(url, enableJavaScript: true) :
    showCustomAlertDialog(context, MultiLanguage.get(CoreLanguageKey.msgNotOpenLink));
  });

  static Future<dynamic> showOptionDialog(BuildContext context, String title,
      final List<dynamic> values, String id, {Color primaryColor = Colors.white}) {
    final double height = 49 * (values.length + 1).sp + (values.length * 2).sp;
    final double max = 1.sh - 20.sp - ScreenUtil().statusBarHeight - ScreenUtil().bottomBarHeight;
    return showDialog(context: context, builder: (context) =>
        Column(mainAxisAlignment: MainAxisAlignment.end, mainAxisSize: MainAxisSize.min,
            children: [
              Container(height: height > max ? max : height,
                  margin: EdgeInsets.fromLTRB(0.1.sw, 0, 0.1.sw, 20.sp),
                  decoration: BoxDecoration(color: Colors.white, borderRadius: BorderRadius.circular(10.sp)),
                  child: ListView(padding: EdgeInsets.zero, children: createItems(context, title, values, id, primaryColor)))
            ]));
  }

  static List<Widget> createItems(BuildContext context, String title,
      List<dynamic> values, String id, Color primaryColor) {
    final line = Divider(color: Colors.grey.shade300, height: 1.sp);
    final List<Widget> list = [];
    list.add(Container(alignment: Alignment.center, height: 49.sp,
        child: Text(title, style: TextStyle(
            decoration: TextDecoration.none, fontSize: 18.sp,
            color: Colors.black87, fontWeight: FontWeight.normal))));
    for (var i = 0; i < values.length; i++) {
      list.add(line);
      list.add(ButtonCustomTransparent(() => Navigator.of(context).pop(values[i]),
          Container(height: 48.sp, alignment: Alignment.center,
              child: TextCustom(values[i].name, size: 16.sp,
                  color: id != values[i].id ? primaryColor : Colors.white)),
          color: id != values[i].id ? Colors.transparent : primaryColor,
          borderRadius: i == values.length - 1
              ? BorderRadius.only(bottomLeft: Radius.circular(10.sp), bottomRight: Radius.circular(10.sp))
              : BorderRadius.zero));
    }
    return list;
  }

  static Future<bool?> showCustomAlertDialog(BuildContext context, String message,
      {String? title, bool isActionCancel = false, double elevation = 4.0,
        Color primaryColor = Colors.white54}) {
    final List<Widget> actions = [];
    if (isActionCancel)
      actions.add(addAction(context, MultiLanguage.get(CoreLanguageKey.btnCancel), false, Colors.grey, elevation));

    actions.add(addAction(context, MultiLanguage.get(CoreLanguageKey.btnOK), true, primaryColor, elevation));

    return showDialog(context: context, barrierDismissible: false, barrierColor: Colors.black12,
        builder: (context) => AlertDialog(actionsPadding: EdgeInsets.all(8.sp),
            title: Text(title??MultiLanguage.get(CoreLanguageKey.ttlWarning), style: TextStyle(fontWeight: FontWeight.bold, fontSize: 14.sp)),
            content: Text(message, style: TextStyle(fontSize: 14.sp)), actions: actions));
  }

  static Widget addAction(BuildContext context, String buttonName, value, Color color, double elevation) =>
      ElevatedButton(
          child: Text(buttonName, style: TextStyle(color: Colors.white, fontSize: 14.sp)),
          onPressed: () => Navigator.of(context).pop(value is TextEditingController ? value.text : value),
          style: ButtonStyle(padding: MaterialStateProperty.all<EdgeInsets>(EdgeInsets.all(8.sp)),
              backgroundColor: MaterialStateProperty.all<Color>(color),
              elevation: MaterialStateProperty.resolveWith((states) => elevation))
      );

  static goToRealPage(BuildContext context, String pageName) {
    while (Navigator.of(context).canPop() && Navigator.of(context).widget.runtimeType.toString() !=
        pageName) Navigator.of(context).pop(false);
  }

  static void goBack(BuildContext context, value) => Navigator.of(context).pop(value);

  static void clearAllPages(BuildContext context) {
    while (Navigator.of(context).canPop()) Navigator.of(context).pop(false);
  }

  static void goToPage(BuildContext context, dynamic page,
      {bool clearAllPage = false, bool hasBack = false, Function? callback}) {
    if (hasBack) {
      Navigator.push(context, MaterialPageRoute(builder: (context) => page))
          .then((value) {
        if (callback != null && value != null) callback(value);
      });
    } else {
      if (clearAllPage) clearAllPages(context);
      Navigator.pushReplacement(context, MaterialPageRoute(builder: (context) => page));
    }
  }

  static bool checkResponse(BuildContext context, BaseResponse response, Function logout,
      Color primaryColor, {bool showError = true, bool passString = false}) {
    if (response.checkTimeout()) {
      showCustomAlertDialog(context, MultiLanguage.get(CoreLanguageKey.msgAnotherLogin),
          primaryColor: primaryColor).then((value) => logout(context));
      return false;
    }
    if (response.checkOK(passString: passString)) return true;
    if (showError && response.data != null) showCustomAlertDialog(context, response.data, primaryColor: primaryColor);
    return false;
  }
}